/* 
 * Copyright (c) 2009, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

/*
 *  ======== lpm_dev.h ========
 *
 */


#if !defined (LPM_DEV_H)
#define LPM_DEV_H


#if defined(OS_PROS_DAVINCI)

#include <stdio.h>

#elif defined(OS_LINUX)


#elif defined(WIN32)


#else

    #error "Operating System undefined";

#endif


#include "lpm_base.h"


#if defined(__cplusplus)
extern "C" {
#endif


typedef enum {
    LPM_EXTERNAL_VOLTAGE_OFF_FXN,       /* LPM_ExtPwrOffFxn */
    LPM_EXTERNAL_VOLTAGE_OFF_ARG,
    LPM_EXTERNAL_VOLTAGE_ON_FXN,        /* LPM_ExtPwrOnFxn */
    LPM_EXTERNAL_VOLTAGE_ON_ARG,
} LPM_Action;


/* External Power ON/OFF functions provided by the driver. */
typedef void (*LPM_ExtPwrOffFxn)(void *);
typedef void (*LPM_ExtPwrOnFxn)(void *);

/* Operating System service functions */
typedef LPM_Status (*LPM_RemapFxn)(unsigned long pa,
                                 unsigned long size, unsigned long *va);
typedef LPM_Status  (*LPM_SignalFxn)(void *);
typedef LPM_Status  (*LPM_UnmapFxn)(void *va);
typedef LPM_Status  (*LPM_WaitFxn)(void *);
typedef void        (*LPM_TraceFxn)(char *fmt, ...);

// Device function types.
typedef LPM_Status (*LPM_Command)(void *, LPM_Action, unsigned long value);
typedef LPM_Status (*LPM_ConnectFxn)(void *);
typedef LPM_Status (*LPM_DisonnectFxn)(void *);
typedef LPM_Status (*LPM_OffFxn)(void *);
typedef LPM_Status (*LPM_OnFxn)(void *, int);
typedef LPM_Status (*LPM_ResumeFxn)(void *);
typedef LPM_Status (*LPM_SetPowerStateFxn)(void *, LPM_PowerState);


// LPM Device Structure
typedef struct LPM_Device {

    /* device operational functions */
    LPM_Command                 command;
    LPM_ConnectFxn              connect;
    LPM_DisonnectFxn            disconnect;
    LPM_OffFxn                  off;
    LPM_OnFxn                   on;
    LPM_ResumeFxn               resume;
    LPM_SetPowerStateFxn        setPowerState;

    /* LPM_Device private instance pointer */
    void                       *instance;
} LPM_Device;


typedef struct LPM_DevAttrs {
    LPM_RemapFxn        os_remap;
    LPM_SignalFxn       os_signal;
    LPM_UnmapFxn        os_unmap;
    LPM_WaitFxn         os_wait;
    LPM_TraceFxn        os_trace;
    void               *os_instance;
} LPM_DevAttrs;

LPM_Status LPM_init (int index, LPM_Device *device, LPM_DevAttrs *attrs);
LPM_Status LPM_exit (void *instance);


#if defined(__cplusplus)
}
#endif


#endif /* !defined (LPM_DEV_H) */
/*
 *  @(#) ti.bios.power; 1, 1, 1,1; 7-23-2009 21:10:08; /db/atree/library/trees/power/power-g08x/src/
 */

