/* 
 * Copyright (c) 2009, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */
/*
 *  ======== cmem_test.c ========
 */

#include <windows.h>
#include <kfuncs.h>

#include <ti/sdo/winceutils/cmem/cmem.h>
#include <ti/sdo/winceutils/test/cmemt.h>

#define __D(fmt, ...) RETAILMSG(1, (L"CMEM TEST Debug: " L##fmt, __VA_ARGS__))


/*
 *  ======== main ========
 */
void main()
{
        CMEM_AllocParams  allocParams;
        CMEMT_AllocParams allocTParams;
        PVOID             heapPtr;
        PVOID             heapTPtr;
        PVOID             ptrPA;
        PVOID             ptrTPA;
        PVOID             ptrxPA;
        PVOID             ptrxTPA;
    PVOID             poolBuf;
    PVOID             poolTBuf;

    DebugBreak();

    /* Initialize CMEM and test driver */
        CMEM_init();
    CMEMT_init();

    /* Allocate a buffer from CMEM pool */
        allocParams.type = CMEM_POOL;
        allocParams.flags = CMEM_NONCACHED;
        poolBuf = CMEM_allocPool(0, &allocParams);

    __D("Pool buffer address: %#x.\n", (unsigned int)poolBuf);

    /* Allocate a buffer from the test driver pool */
        allocTParams.type = CMEMT_POOL;
        allocTParams.flags = CMEMT_NONCACHED;
        poolTBuf = CMEMT_allocPool(0, &allocTParams);

    __D("Test pool buffer address: %#x.\n", (unsigned int)poolTBuf);

    /* Allocate buffer from CMEM heap */
        allocParams.type = CMEM_HEAP;
        heapPtr = CMEM_alloc(1024, &allocParams);

    __D("Heap buffer address: %#x.\n", (unsigned int)heapPtr);

    /* Allocate buffer from test driver heap */
        allocTParams.type = CMEMT_HEAP;
        heapTPtr = CMEMT_alloc(1024, &allocTParams);

    __D("Test heap buffer address: %#x.\n", (unsigned int)heapTPtr);

    /*
     *  Determine physical addresses of buffers. These calls should just
     *  use the CMEM and CMEMT book keeping.
     */
        ptrPA = (PVOID)CMEM_getPhys(heapPtr);
    ptrTPA = (PVOID)CMEMT_getPhys(heapTPtr);

    /*
     *  Determine physical addresses of buffers allocated from one driver
     *  using the getPhys() of the other driver. These calls should use
     *  the Windows CE LockPages() API to get the physical address.
     */
        ptrxPA = (PVOID)CMEMT_getPhys(heapPtr);
    ptrxTPA = (PVOID)CMEM_getPhys(heapTPtr);

    /* See if the results are the same */
    __D("ptrPA: 0x%x, ptrxPA: 0x%x\n", ptrPA, ptrxPA);
    __D("ptrTPA: 0x%x, ptrxTPA: 0x%x\n", ptrTPA, ptrxTPA);

    /*
     *  Now try offsets.
     */
        ptrxPA = (PVOID)CMEMT_getPhys((PVOID)((unsigned long)heapPtr + 10));
    ptrxTPA = (PVOID)CMEM_getPhys((PVOID)((unsigned long)heapTPtr + 10));

    /* See if the results are the same */
    __D("ptrPA: 0x%x, ptrxPA: 0x%x\n", ptrPA, ptrxPA);
    __D("ptrTPA: 0x%x, ptrxTPA: 0x%x\n", ptrTPA, ptrxTPA);

        CMEM_exit();
    CMEMT_exit();
}
/*
 *  @(#) ti.sdo.winceutils.cmem; 1, 0, 0,41; 9-4-2009 14:25:36; /db/atree/library/trees/winceutils/winceutils-a12x/src/
 */

