/* 
 * Copyright (c) 2009, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

/*
 *  ======== Package.close ========
 *  Close this package; optionally (re)set other config parameters in the
 *  model
 */
function close()
{
    trace(this.$name + ".close() ...");
    var settings = this.Settings;
    var prog = Program;

    /* if undefined, take profile from the Settings module */
    if (this.profile == undefined) {
        this.profile = xdc.useModule('ti.sdo.ce.global.Settings').profile;
    }

//    trace(this.$name + ".close() useLinkArbiter: " +settings.useLinkArbiter);

    var global = xdc.useModule('ti.sdo.ce.osal.Global');

    /*
     *  Establish dependency on OS package. The name of the OS package is
     *  assumed to be the full path name of the os up to the os module
     *  name (eg, for 'ti.sdo.ce.osal.bios.Settings', it is
     *  'ti.sdo.ce.osal.bios').
     *  This affects the order that libraries appear in the .xdl file. If
     *  we don't have this, for a remote app, osal_linux_470.a470MV will
     *  be before ipc_dsplink_6446.a470MV. However, adding the following
     *  line to ipc.dsplink.package.close() fixed this:
     *      xdc.useModule('ti.sdo.ce.osal.linux.Settings');
     */
    var osPkg = null;
    if (global.os != undefined) {
        osPkg = global.os.$name;
        osPkg = osPkg.substring(0, osPkg.lastIndexOf("."));
        //print("os package name: " + osPkg);
        //xdc.loadPackage(osPkg);
    }

    // NOTE: if CE is used, and settings.ipc was not set by the app, then
    // CE package.close() would have set it (CE depends on ipc, since
    // CERuntime_init() calls Processor_init() and Comm_init()).

    /* For backwards compatibility (CE 1.20 and CE 2.00) */
    if ((settings.ipc == undefined) && (settings.commType == undefined) &&
            (global.runtimeEnv != undefined)) {
        warn(this.$name + ".close(): WARNING: ipc undefined, using " +
                "deprecated osal.global.runtimeEnv to set ipc");

        //
        //  Assume this in on old app.cfg that has configured the OS through
        //  osal.Global.runtimeEnv, and use that to set ipc.
        //
        if (global.runtimeEnv == global.DSPLINK_LINUX) {
            // This may not work. In some cases, ipc.dsplink.close()
            // is happening before ipc.close() (eg, if the app .cfg file
            // uses the dsplink.Ipc module.)
            trace(this.$name + ".close(): setting ipc to dsplink");
            var dsplinkIpc = xdc.useModule('ti.sdo.ce.ipc.dsplink.Ipc');
            settings.ipc = dsplinkIpc;
            osPkg = 'ti.sdo.ce.osal.linux';

            dsplinkIpc.armCommMsgSize = global.armCommMsgSize;
            dsplinkIpc.armCommNumMsgs = global.armCommNumMsgs;
            dsplinkIpc.commLocateRetries = global.commLocateRetries;
            dsplinkIpc.useLinkArbiter = global.useLinkArbiter;
            if (global.usePowerManagement != undefined) {
                dsplinkIpc.usePowerManagement = global.usePowerManagement;
            }
        }
        else if ((global.runtimeEnv == global.DSPLINK_WINCE)) {
            // There is currently no WINCE ipc, only DSPLINK_WINCE
            trace(this.$name + ".close(): setting ipc to dsplink");
            var dsplinkIpc = xdc.useModule('ti.sdo.ce.ipc.dsplink.Ipc');
            settings.ipc = dsplinkIpc;
            osPkg = 'ti.sdo.ce.osal.wince';

            dsplinkIpc.armCommMsgSize = global.armCommMsgSize;
            dsplinkIpc.armCommNumMsgs = global.armCommNumMsgs;
            dsplinkIpc.commLocateRetries = global.commLocateRetries;
            dsplinkIpc.useLinkArbiter = global.useLinkArbiter;
            if (global.usePowerManagement != undefined) {
                dsplinkIpc.usePowerManagement = global.usePowerManagement;
            }
        }
        else if ((global.runtimeEnv == global.DSPLINK_BIOS) ||
                (global.runtimeEnv == global.DSPBIOS)) {

            trace(this.$name + ".close(): setting ipc to bios");
            var biosIpc = xdc.useModule('ti.sdo.ce.ipc.bios.Ipc');
            settings.ipc = biosIpc;

            if (global.runtimeEnv == global.DSPLINK_BIOS) {
                var link = xdc.useModule('ti.sdo.ce.ipc.dsplink.dsp.Settings');
            }
            osPkg = 'ti.sdo.ce.osal.bios';

            biosIpc.armCommMsgSize = global.armCommMsgSize;
            biosIpc.dspConfigureMsgqAndPool = global.dspConfigureMsgqAndPool;
            biosIpc.dspUseMsgqAndPool = global.dspUseMsgqAndPool;
            biosIpc.dspCommNumMsgQueues = global.dspCommNumMsgQueues;
            biosIpc.dspCommUseExactMatch = global.dspCommUseExactMatch;
            biosIpc.dspCommMsgSize = global.dspCommMsgSize;
            biosIpc.dspCommNumMsgs = global.dspCommNumMsgs;
            if (global.usePowerManagement != undefined) {
                biosIpc.usePowerManagement = global.usePowerManagement;
            }
            biosIpc.powerSaveMemoryBlockName = global.powerSaveMemoryBlockName;
            trace(this.$name + ".close(): Done setting ipc to bios");
        }
        else if (global.runtimeEnv == global.LINUX) {
            trace(this.$name + ".close(): setting ipc to linux");
            var linuxIpc = xdc.useModule('ti.sdo.ce.ipc.linux.Ipc');
            settings.ipc = linuxIpc;
            osPkg = 'ti.sdo.ce.osal.linux';
        }
        else if (global.runtimeEnv == global.NONE) {
            trace(this.$name + ".close(): setting ipc to noOS");
            var noOSIpc = xdc.useModule('ti.sdo.ce.ipc.noOS.Ipc');
            settings.ipc = noOSIpc;
            osPkg = 'ti.sdo.ce.osal.noOS';
        }
        else if (global.runtimeEnv == global.WINCE) {
            trace(this.$name + ".close(): setting ipc to noOS");
            var noOSIpc = xdc.useModule('ti.sdo.ce.ipc.noOS.Ipc');
            settings.ipc = noOSIpc;
            osPkg = 'ti.sdo.ce.osal.wince';
        }
    }

    // If settings.ipc is undefined, just leave it. Osal.Global has a
    // dependency in Ipc, since it uses the type ipc.IIpc.ArmDspLinkConfig
    // for its DEFAULT_ARMDSPLINKCONFIG, which is need for backwards
    // compatibility with CE 1.20. However, if we set settings.ipc to
    // one of the ipcs (eg ipc.linux), that will bring in the ipc library,
    // even when not used (eg, an osal memory test). Therefore, don't set
    // the ipc, to make it easier for osal porters to build osal tests
    // that don't require ports or re-builds of an ipc library.
    if (settings.ipc == undefined) {
        // Don't throw an exception, but may want to print warning.
        warn(this.$name + ": ipc undefined");
    }

    trace(this.$name + ".close() done!");
}

/*
 *  ======== Package.validate ========
 *  Verify the config params for this package; called after close()
 */
function validate()
{
    trace(this.$name + ".validate() ...");

}

/*
 *  ======== getLibs ========
 */
function getLibs(prog)
{
    return (null);
}


/*
 *  ======== warn ========
 *  For warnings that we may not really want to print.
 */
function warn(msg)
{
    //print("WARN:  " + msg);
}

/*
 *  ======== trace ========
 */
function trace(msg)
{
    //print("TRACE:  " + msg);
}
/*
 *  @(#) ti.sdo.ce.ipc; 2, 0, 1,140; 12-18-2009 17:42:52; /db/atree/library/trees/ce/ce-o06x/src/
 */

