/* 
 * Copyright (c) 2009, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

var xmlOps = xdc.loadCapsule("xmlOps.xs");
var shared_gui = xdc.loadCapsule("shared_gui.xs");
var wizard_constants = xdc.loadCapsule("wizard_constants.xs");
var mkpkg = xdc.module("xdc.tools.mkpkg.Main");
var csd = String(xdc.csd()).replace(/\\/g,"/");

var memComment = [];
memComment['RESET_VECTOR'] =
    'RESET_VECTOR: off-chip memory for the reset vector table';
memComment['DSPLINKMEM'] =
    'DSPLINK: off-chip memory reserved for DSPLINK code and data';
memComment['DDRALGHEAP'] =
    'DDRALGHEAP: off-chip memory for dynamic algmem allocation';
memComment['DDR2'] =
    'DDR2: off-chip memory for application code and data';

function defaultMemConfig(platform)
{
    var memConfig = [];
    if (platform == 'ti.platforms.evm3530') {
        memConfig = [
            { sect : 'RESET_VECTOR', base : '0x87300000', len : '0x00000080'},
            { sect : 'DSPLINKMEM',   base : '0x87301000', len : '0x000FF000'},
            { sect : 'DDRALGHEAP',   base : '0x87400000', len : '0x00900000'},
            { sect : 'DDR2',         base : '0x87D00000', len : '0x00300000'}
        ];
    }
    if (platform == 'ti.platforms.evmDM6446') {
        memConfig = [
            { sect : 'DDRALGHEAP',   base : '0x88000000', len : '0x07A00000'},
            { sect : 'DDR2',         base : '0x8FA00000', len : '0x00400000'},
            { sect : 'DSPLINKMEM',   base : '0x8FE00000', len : '0x00100000'},
            { sect : 'RESET_VECTOR', base : '0x8FF00000', len : '0x00000080'}
        ];
    }
    if (platform == 'ti.platforms.evmDM6467') {
        memConfig = [
            { sect : 'DDRALGHEAP',   base : '0x88000000', len : '0x07A00000'},
            { sect : 'DDR2',         base : '0x8FA00000', len : '0x00400000'},
            { sect : 'DSPLINKMEM',   base : '0x8FE00000', len : '0x00100000'},
            { sect : 'RESET_VECTOR', base : '0x8FF00000', len : '0x00000080'}
        ];
    }
    if (platform == 'ti.platforms.evmOMAPL137') {
        memConfig = [
            { sect : 'DDR2',         base : '0xC2C00000', len : '0x00300000'},
            { sect : 'RESET_VECTOR', base : '0xC2F00000', len : '0x00000080'},
            { sect : 'DSPLINKMEM',   base : '0xC2F01000', len : '0x000FF000'},
            { sect : 'DDRALGHEAP',   base : '0xC3000000', len : '0x01000000'}

        ];
    }
    if (platform == 'ti.platforms.evmOMAPL138') {
        memConfig = [
            { sect : 'DDR2',         base : '0xC2C00000', len : '0x00300000'},
            { sect : 'RESET_VECTOR', base : '0xC2F00000', len : '0x00000080'},
            { sect : 'DSPLINKMEM',   base : '0xC2F01000', len : '0x000FF000'},
            { sect : 'DDRALGHEAP',   base : '0xC3000000', len : '0x01000000'}

        ];
    }
    return (memConfig);
}

function defaultMem()
{
    var def = [];
    def['ti.platforms.evm3530'] = true;
    def['ti.platforms.evmDM6446'] = true;
    def['ti.platforms.evmDM6467'] = true;
    def['ti.platforms.evmOMAPL137'] = true;
    def['ti.platforms.evmOMAPL138'] = true;
    return (def);
}

function argsValidation(serverEnv) {
  
    if ( serverEnv.xmlFile != null && serverEnv.xmlFile != "null") {
        if ( java.io.File( serverEnv.xmlFile ).exists() ) {
            try {
                serverEnv = xmlOps.loadXml( serverEnv.xmlFile, serverEnv );
            }
            catch(e) { 
                gprint("Error: " + e, serverEnv ); 
            }
        }
        else { 
            gprint("XML File, " + serverEnv.xmlFile + ", not found. "
                + "Wizard starting without loading XML File...", 
                serverEnv); 
            serverEnv.xmlFile = "null";
        }
    }
    if (!serverEnv.platform) {
        serverEnv.platform = 'ti.platforms.evmDM6446';
    }
    if (!serverEnv.serverName) {
        serverEnv.serverName = 'mycompany.mygroup.servers.servercom';
    }

    if (serverEnv.output == null || serverEnv.output == 'null') {
        serverEnv.output = wizard_constants.ENTER_OUTREPO;
    }

    if (!serverEnv.gui) { 
        validateGroups(serverEnv); /* for xml file backward compatibility */
    }
    if (!serverEnv.memConfig || !serverEnv.memConfig[serverEnv.platform]) {
        if (!serverEnv.memConfig) {
            serverEnv.memConfig = [];
        }
        serverEnv.memConfig[serverEnv.platform] = defaultMemConfig(serverEnv.platform);
        if (!serverEnv.defaultMem) {
            serverEnv.defaultMem = [];
        }
        //serverEnv.defaultMem[serverEnv.platform] = true;
        serverEnv.defaultMem[serverEnv.platform] = false;
    }
    serverEnv.memComment = memComment;
    if (!serverEnv.memAlign || !serverEnv.memAlign[serverEnv.platform]) {
        if (!serverEnv.memAlign) {
            serverEnv.memAlign = [];
        }
        //serverEnv.memAlign[serverEnv.platform] = 'none';
        serverEnv.memAlign[serverEnv.platform] = 'bottom';
    }

    if (!serverEnv.cgTools) {
        serverEnv.cgTools = wizard_constants.ENTER_CGTOOLS
    }

    if (serverEnv.genCCSProj != true && serverEnv.genCCSProj != false) {
        serverEnv.genCCSProj = true;
    }

    return serverEnv;
}

function gprint(message, serverEnv) {
    if (serverEnv.gui == true) {
        shared_gui.createMessageBox(message, serverEnv);
    }
    else{
        print(message);
    }
}

function createFiles(serverEnv) {

    var status = 0;
    serverEnv = argsValidation(serverEnv);

    /* find configs within codecs */
    serverEnv = getConfigsFromModules(serverEnv);

    var outputDir = "" + (serverEnv.output+"") + "/" 
        + (serverEnv.serverName+"").replace(/\./g,"/");
    outputDir = outputDir.replace("//", "/");
    outputDir = outputDir.replace(/\\/g,"/");
    
    if ( java.io.File( outputDir ).exists() ) {
        var list = java.io.File( outputDir ).list();
        if (list.length!=0) {
            if (!serverEnv.gui ) {
                gprint("\nWarning: " + outputDir + " is not empty--files may be "
                    + "overwritten... Would you like to continue? y/n", 
                    serverEnv);
                var isr = new java.io.InputStreamReader( 
                    java.lang.System["in"] );
                var br = new java.io.BufferedReader( isr );
                
                try{
                    var output = br.readLine();
                }
                catch(e) {}
    
                if (output == "n") {
                    gprint("Quitting now...", serverEnv);
                    status = 1;
                }
                else if(output != "y") {
                    gprint("Invalid selection. Quitting now...", serverEnv);
                    status = 1;
                }
            }
            if (serverEnv.gui) {
                var i = shared_gui.createYesNoBox( "\nWarning: " + outputDir 
                    + " is not empty--files may be overwritten. "
                    + "Would you like to continue?", serverEnv);
                if (i != 1) {
                    status = 1;
                }
            }
        }
    } 

    if (status == 0) {    
        /* function mkpkg( template_dir, output, args ) */
        try {
            mkpkg.mkpkg("ti/sdo/ce/wizards/genserver/templates", outputDir, 
                        serverEnv);

            if (serverEnv.genCCSProj) {
                mkpkg.mkpkg("ti/sdo/ce/wizards/genserver/templates/ccsv4", outputDir,
                            serverEnv);
            }
        }
        catch (e) {
            gprint("An error while creating package " +
                    serverEnv.serverName + ":\n" + e, serverEnv);
            status = 1;
        }
    }
    return(status);
}

/**
* Parses ICodec using xdc.services.spec
* @return Declaration nodes sorted by kind
*/ 
function getConfigsFromModules(serverEnv ) {

    serverEnv.configs = [];
    serverEnv.cePackages = [];
    serverEnv.ceModules = [];
    serverEnv.idma3Fxns = false;

    try {
        /* create a BrowserSession with a private package path */
        var env = new Packages.xdc.services.global.Env();
        var bs = Packages.xdc.services.spec.BrowserSession(serverEnv.searchPathArr, env);
    }
    catch (e) {
        gprint("Error creating browser session\n" + e, serverEnv);
        configsObj = {};
        serverEnv.configs.push(configsObj);      
        return (serverEnv);
    }

    for (var i = 0; i < serverEnv.cfgCodecs.length; i++) {
        try {
            var unit = bs.loadUnit(serverEnv.cfgCodecs[i]);
            if (unit == null) {
                gprint("Error loading module " + serverEnv.cfgCodecs[i],
                       serverEnv);
                continue;
            }

            var configsObj = {};
            var configNames = [];
            var configTypes = [];
            var configValues = [];
            var declList = unit.getDecls();
        
            for (var j = 0; j < declList.size(); j++) {
                var decl = declList.get(j);
                var kind = String(decl.getXmlTag());
                var name = String(decl.getName());
                if (!serverEnv.idma3Fxns && name == 'idma3Fxns') {
                    if (String(Packages.xdc.services.spec.Expr.toText(
                               decl.getInit())).length) {
                        serverEnv.idma3Fxns = true;
                    }
                }
                var readonly = String(decl.isReadonly());
                if (kind == "config" && (readonly == "false" || name == "alg")) {
                    if (name == 'uuid' || name == 'isLocal') {
                        continue;
                    }
                    var isAlg = name == "alg";
                    var configs = unit.getConfigs();
                    var bspec = _find(configs, serverEnv.cfgCodecs[i] + '.' +
                                      name);
                    if (!isAlg) {
                        configNames.push(name);
                    }
                    if (bspec != undefined) {
                        var typename =  String(_getTypeName(bspec)); // returns Bool, String
                        if (!isAlg) {
                            configTypes.push(typename);
                        }
                    }
                    else if (!isAlg) {
                        configTypes.push('');
                    }
                    if (isAlg) { /* get configs from alg module */
                        var algMod = typename.substr(0, typename.lastIndexOf(':Module'));
                        var algUnit = bs.loadUnit(algMod);
                        if (algUnit == null) {
                            gprint("Error loading module " + algUnit, serverEnv);
                        }
                        else {
                            var algDeclsList = algUnit.getDecls();
                            for (var k = 0; k < algDeclsList.size(); k++) {
                                var algDecl = algDeclsList.get(k);
                                kind = String(algDecl.getXmlTag());
                                readonly = String(algDecl.isReadonly());
                                if (kind == "config" && readonly == "false") {
                                    name = String(algDecl.getName());
                                    configs = unit.getConfigs();
                                    bspec = _find(configs, algMod + '.' + name);
                                    configNames.push('alg.' + name);
                                    if (bspec != undefined) {
                                        typename =  String(_getTypeName(bspec));
                                        configTypes.push(typename);
                                    }
                                    else {
                                        configTypes.push('');
                                    }
                                    configValues.push(String(Packages.xdc.
                                            services.spec.Expr.
                                            toText(algDecl.getInit())));
                                }
                            }
                        }
                        serverEnv.cePackages.push(algMod.substr(0, algMod.lastIndexOf('.')));
                        serverEnv.ceModules.push(algMod.substr(algMod.lastIndexOf('.') + 1));
                    }
                    else {
                        configValues.push(String(Packages.xdc.services.spec.
                                          Expr.toText(decl.getInit())));
                    }
                }
            }
            configsObj.configNames = configNames;
            configsObj.configTypes = configTypes;
            configsObj.configValues = configValues;
        }
        catch(e) {
            gprint("Error finding config params from module: " +
                   serverEnv.cfgCodecs[i] + "\n" + e, serverEnv);

            configsObj = {};
        }

        serverEnv.configs.push(configsObj);      
    }    
    return serverEnv;
}

function _getTypeName(spec) {
    var type = spec.getType().xmlsig();
    var n = type;
        if (type != null) {
            if (type.startsWith("T!") || type.startsWith("K!")) {
                var s1 = type.split("!");
                n = s1[1];
            }
        }
    return(n);
}

 

/**
 * Return the named element from the list.
 * 
 * @param list List of Node-derived objects
 * @param name of object to retrieve
 * @return Node named object if found, null if not
 */

function _find(list, name) {
    var o = null;
    var it = list.iterator();
    while (it.hasNext()) {
        var n = it.next();
        if (n.getName().compareTo(name) == 0) {
            o = n;
            break;
        } else if (n.getQualName().compareTo(name.replace('\/', '.')) == 0) {
            o = n;
            break;
        }
    }
    return (o);
}


function validateGroups(serverEnv)
{
    for (var i = 0; i < serverEnv.cfgCodecs.length; i++) {
        var qn = serverEnv.cfgCodecs[i];
        var groupId = serverEnv.algVals[qn].groupId;
        if (!serverEnv.groups[groupId]) {
            defaultGroup(serverEnv, groupId);
        }
    }
}


function defaultGroup(serverEnv, groupId)
{
    serverEnv.groups[groupId] = {};
    serverEnv.groups[groupId].dskt2 = {};
    serverEnv.groups[groupId].edma3 = {};
    serverEnv.groups[groupId].dman3 = {};
    platformSpecificDefault(serverEnv.groups[groupId], groupId)

    if (groupId == wizard_constants.VIDEO_GROUPID) {

        serverEnv.groups[groupId].dskt2.saram = '32768';

        /* ti.platforms.evmDM6467 */
        serverEnv.groups[groupId].edma3.maxPaRams = '384';
        serverEnv.groups[groupId].edma3.maxTccs = '26';
        serverEnv.groups[groupId].edma3.maxEdmaChannels = '24';
        serverEnv.groups[groupId].edma3.maxQdmaChannels = '4';

        /* all other */
        serverEnv.groups[groupId].dman3.numPaRam = '48';
    }
    else {
        if (groupId == wizard_constants.AUDIO_GROUPID) {
            serverEnv.groups[groupId].dskt2.saram = '8192';
        }
        else {
            serverEnv.groups[groupId].dskt2.saram = '0';
        }

        /* ti.platforms.evmDM6467 */
        serverEnv.groups[groupId].edma3.maxPaRams = '0';
        serverEnv.groups[groupId].edma3.maxTccs = '0';
        serverEnv.groups[groupId].edma3.maxEdmaChannels = '0';
        serverEnv.groups[groupId].edma3.maxQdmaChannels = '0';

        /* all other */
        serverEnv.groups[groupId].dman3.numPaRam = '0';
    }
}


function platformSpecificDefault(obj, groupId)
{
    var daram = [];
    var numTccGroup = [];
    if (groupId == wizard_constants.VIDEO_GROUPID) {
        daram['ti.platforms.evm3530'] = '65536';
        daram['ti.platforms.evmDM6446'] = '32768';
        daram['ti.platforms.evmDM6467'] = '32768';
        daram['ti.platforms.evmOMAPL137'] = '32768';
        daram['ti.platforms.evmOMAPL138'] = '32768';

        numTccGroup['ti.platforms.evm3530'] = '32';
        numTccGroup['ti.platforms.evmDM6446'] = '32';
        numTccGroup['ti.platforms.evmDM6467'] = '32';
        numTccGroup['ti.platforms.evmOMAPL137'] = '16';
        numTccGroup['ti.platforms.evmOMAPL138'] = '16';
    }
     else {
        if (groupId == wizard_constants.AUDIO_GROUPID) {
            daram['ti.platforms.evm3530'] = '8192';
            daram['ti.platforms.evmDM6446'] = '8192';
            daram['ti.platforms.evmDM6467'] = '8192';
            daram['ti.platforms.evmOMAPL137'] = '8192';
            daram['ti.platforms.evmOMAPL138'] = '8192';
        }
        else {
            daram['ti.platforms.evm3530'] = '0';
            daram['ti.platforms.evmDM6446'] = '0';
            daram['ti.platforms.evmDM6467'] = '0';
            daram['ti.platforms.evmOMAPL137'] = '0'
            daram['ti.platforms.evmOMAPL138'] = '0'
        }
        numTccGroup['ti.platforms.evm3530'] = '0';
        numTccGroup['ti.platforms.evmDM6446'] = '0';
        numTccGroup['ti.platforms.evmDM6467'] = '0';
        numTccGroup['ti.platforms.evmOMAPL137'] = '0';
        numTccGroup['ti.platforms.evmOMAPL138'] = '0';
    }
    obj.dskt2.daram = daram;
    obj.dman3.numTccGroup = numTccGroup;
}


/*
 *  @(#) ti.sdo.ce.wizards.genserver; 1, 0, 0,42; 12-18-2009 17:46:07; /db/atree/library/trees/ce/ce-o06x/src/
 */

